// ignore_for_file: implementation_imports, unnecessary_import
import 'package:meta/meta.dart';

abstract class AggregateResults {
  final Map<Uri, Set<String>> _sourceMap = {};
  abstract final String path;

  @mustCallSuper
  void merge(covariant AggregateResults other) {
    for (final kv in other._sourceMap.entries) {
      final src = kv.key;
      final nameList = kv.value;

      if (_sourceMap.containsKey(src)) {
        _sourceMap[src]!.addAll(nameList);
      } else {
        _sourceMap[src] = nameList;
      }
    }
  }

  @override
  String toString();
  void addSourceMap(Uri src, String name) {
    if (_sourceMap.containsKey(src)) {
      _sourceMap[src]!.add(name);
    } else {
      _sourceMap[src] = {name};
    }
  }
}

class AggregateResultsGroup<T extends AggregateResults> {
  final Map<String, T> _groups;
  final String rootPackage;
  final String outputPath;
  final Set<String>? order;
  AggregateResultsGroup({
    Map<String, T>? groups,
    required this.rootPackage,
    required this.outputPath,
    required this.order,
  }) : _groups = groups ?? <String, T>{};

  Map<String, T> get groups => _groups;
  void addGroup(String group, T result) {
    if (_groups.containsKey(group)) {
      _groups[group]!.merge(result);
    } else {
      _groups[group] = result;
    }
  }

  void addAll(Map<String, T>? all) {
    if (all == null) return;
    for (final kv in all.entries) {
      addGroup(kv.key, kv.value);
    }
  }

  Map<Uri, Set<String>> mergeSourceMap() {
    final sourceMap = <Uri, Set<String>>{};
    for (final kv in _groups.entries) {
      final results = kv.value;
      final srcMap = results._sourceMap;
      for (final srcPair in srcMap.entries) {
        final source = srcPair.key;
        final nameList = srcPair.value;
        if (sourceMap.containsKey(source)) {
          sourceMap[source]!.addAll(nameList);
        } else {
          sourceMap[source] = nameList;
        }
      }
    }
    return sourceMap;
  }

  @override
  String toString() => throw UnimplementedError();
}
