import 'package:initializer_annotation/initializer_annotation.dart';

/// settings for initializer
class Settings {
  /// save the config of [Initializer].
  final ClassConfig config;

  /// Creates a new [Settings] instance.
  /// If `config` arg is null, will create from default settings [ClassConfig.defaults].
  Settings({
    Initializer? config,
  }) : config = config != null
            ? ClassConfig.fromInitializer(config)
            : ClassConfig.defaults;
}

/// Use class [Initializer] for config in `build.yaml`.
/// This class should have all valid setting in [Initializer].
class ClassConfig {
  /// [Initializer.debug]
  final bool debug;

  /// [Initializer.group]
  final String group;

  /// [Initializer.outputPath]
  final String outputPath;

  /// [Initializer.order]
  final Set<String> order;

  /// Creates a new [ClassConfig] instance.
  const ClassConfig({
    required this.debug,
    required this.group,
    required this.outputPath,
    required this.order,
  });

  /// Creates a new [ClassConfig] instance using a [Initializer] instance.
  factory ClassConfig.fromInitializer(Initializer config) =>
      // #CHANGE WHEN UPDATING initializer_annotation
      ClassConfig(
        debug: config.debug,
        group: config.group ?? defaults.group,
        outputPath: config.outputPath ?? defaults.outputPath,
        order: config.order ?? defaults.order,
      );

  /// default values of ClassConfig
  static const defaults = ClassConfig(
    debug: false,
    group: 'default',
    outputPath: 'lib/src/init.init.dart',
    order: <String>{},
  );

  /// convert to a [Initializer] instance.
  Initializer toInitializer() => Initializer(group: group);
}
